@@ -21,6 +21,7 @@ extern "C" {
 /** Index that concatenates the results from several sub-indexes
  */
 FAISS_DECLARE_CLASS_INHERITED(IndexShards, Index)
+FAISS_DECLARE_DESTRUCTOR(IndexShards)
 
 FAISS_DECLARE_GETTER_SETTER(IndexShards, int, own_fields)
 FAISS_DECLARE_GETTER_SETTER(IndexShards, int, successive_ids)
@@ -35,6 +36,8 @@ int faiss_IndexShards_new_with_options(
 
 int faiss_IndexShards_add_shard(FaissIndexShards* index, FaissIndex* shard);
 
+int faiss_IndexShards_remove_shard(FaissIndexShards* index, FaissIndex* shard);
+
 FaissIndex* faiss_IndexShards_at(FaissIndexShards* index, int i);
 
 #ifdef __cplusplus
