@@ -10,14 +10,15 @@
 // I/O code for indexes
 
 #include "clone_index_c.h"
-#include "clone_index.h"
+#include <faiss/clone_index.h>
 #include "macros_impl.h"
 
 using faiss::Index;
 
-int faiss_clone_index (const FaissIndex *idx, FaissIndex **p_out) {
+int faiss_clone_index(const FaissIndex* idx, FaissIndex** p_out) {
     try {
         auto out = faiss::clone_index(reinterpret_cast<const Index*>(idx));
         *p_out = reinterpret_cast<FaissIndex*>(out);
-    } CATCH_AND_HANDLE
-}
\ No newline at end of file
+    }
+    CATCH_AND_HANDLE
+}
