@@ -10,6 +10,8 @@
 
 #include "AuxIndexStructures_c.h"
 #include <faiss/impl/AuxIndexStructures.h>
+#include <faiss/impl/DistanceComputer.h>
+#include <faiss/impl/IDSelector.h>
 #include <iostream>
 #include "../macros_impl.h"
 
