@@ -54,7 +54,9 @@ struct IndexLattice : Index {
             const float* x,
             idx_t k,
             float* distances,
-            idx_t* labels) const override;
+            idx_t* labels,
+            const SearchParameters* params = nullptr) const override;
+
     void reset() override;
 };
 
