@@ -16,6 +16,7 @@ namespace faiss {
 struct Index;
 struct IndexIVF;
 struct VectorTransform;
+struct Quantizer;
 
 /* cloning functions */
 Index* clone_index(const Index*);
@@ -30,4 +31,6 @@ struct Cloner {
     virtual ~Cloner() {}
 };
 
+Quantizer* clone_Quantizer(const Quantizer* quant);
+
 } // namespace faiss
