@@ -43,6 +43,13 @@ struct VisitedTable;
 struct DistanceComputer; // from AuxIndexStructures
 struct HNSWStats;
 
+struct SearchParametersHNSW : SearchParameters {
+    int efSearch = 16;
+    bool check_relative_distance = true;
+
+    ~SearchParametersHNSW() {}
+};
+
 struct HNSW {
     /// internal storage of vectors (32 bits: this is expensive)
     typedef int storage_idx_t;
@@ -188,30 +195,26 @@ struct HNSW {
             std::vector<omp_lock_t>& locks,
             VisitedTable& vt);
 
-    int search_from_candidates(
-            DistanceComputer& qdis,
-            int k,
-            idx_t* I,
-            float* D,
-            MinimaxHeap& candidates,
-            VisitedTable& vt,
-            HNSWStats& stats,
-            int level,
-            int nres_in = 0) const;
-
-    std::priority_queue<Node> search_from_candidate_unbounded(
-            const Node& node,
-            DistanceComputer& qdis,
-            int ef,
-            VisitedTable* vt,
-            HNSWStats& stats) const;
-
-    /// search interface
+    /// search interface for 1 point, single thread
     HNSWStats search(
             DistanceComputer& qdis,
             int k,
             idx_t* I,
             float* D,
+            VisitedTable& vt,
+            const SearchParametersHNSW* params = nullptr) const;
+
+    /// search only in level 0 from a given vertex
+    void search_level_0(
+            DistanceComputer& qdis,
+            int k,
+            idx_t* idxi,
+            float* simi,
+            idx_t nprobe,
+            const storage_idx_t* nearest_i,
+            const float* nearest_d,
+            int search_type,
+            HNSWStats& search_stats,
             VisitedTable& vt) const;
 
     void reset();
